(*************************************************************************
 *                                                                       *
 *                    XI Olimpiada Informatyczna                         *
 *                                                                       *
 *   Rozwizanie zadania: Sznurki                                        *
 *   Plik:                szn2.ml                                        *
 *   Autor:               Marcin Kubica                                  *
 *************************************************************************)

open Scanf;;
open Array;;
open List;;
let id x = x;;
let inf = 100000;;

(* Typ reprezentujcy dowolne drzewo ukorzenione. *)
type drzewo = Wezel of drzewo list;;

(* Wczytanie danych. Wynikiem jest drzewo i liczba sznurkw. *)
let wczytaj_dane () = 
  (* Liczba wzw. *)
  let n = scanf "%d " id
  in 
    let sasiedzi = make (n+1) []
    and odwiedzone = make (n+1) false
    and konce = ref 0
    in 
      (* Wczytanie krawdzi. *)
      (* Wynikiem jest korzen - jeden z lici. *)
      let rec wczytaj () = 
	let i = ref 1 
	in
	  begin
	    (* Wczytanie krawdzi. *)
	    for j = 1 to n-1 do 
	      let a = scanf "%d " id
	      and b = scanf "%d " id
	      in begin
		sasiedzi.(a) <- b::sasiedzi.(a);
		sasiedzi.(b) <- a::sasiedzi.(b)
	      end
	    done;
	    (* Koce sznurkw. *)
	    for j = 1 to n do 
	      if length sasiedzi.(j) mod 2 = 1 then konce := !konce + 1
	    done; 
	    (* Wybr korzenia -- jeden z lici. *)
	    while length sasiedzi.(!i) > 1 do i:= !i + 1 done;
	    !i 
	  end

      (* Konstrukcja drzewa - DFS. *)
      and dfs v = 
	begin
	  odwiedzone.(v) <- true;
	  Wezel  
	    (fold_left
	      (fun a w -> if odwiedzone.(w) then a else (dfs w)::a)
	      [] sasiedzi.(v))
	end

      in 
        let korzen = wczytaj ()
	in (dfs korzen, !konce / 2);;
        

(* Wybiera z listy fragment od i-tego do j-tego elementu. *)
let cut i j l = 
  let rec cut_iter i j l a = 
    if i > 1 then 
      cut_iter (i-1) (j-1) (tl l) a
    else if j > 0 then 
      cut_iter i (j-1) (tl l) (hd l :: a)
    else 
      rev a
  in
    cut_iter i j l [];;


(* Sprawdza ile pierwszych elementw z obu list *)
(* mona poczy w pary o sumach <= d.         *)
let ile_par l1 l2 d = 
  let rec ile_par_iter l1 l2 a = 
    match (l1, l2) with
      ([], []) -> a | 
      (h1::t1, h2::t2) -> 
	if h1 + h2 + 2 <= d then 
	  ile_par_iter t1 t2 (a+1)
	else a |
      _ -> a
  in 
    ile_par_iter l1 l2 0;;
    
(* Sprawd czy elementy listy posortowanej *)
(* nierosnco mona poczy w pary <= d   *)
let pary l d = 
  let dl = length l
  in 
    if dl mod 2 = 0 then 
      ile_par (cut 1 (dl / 2) l) (rev (cut (dl / 2 + 1) dl l)) d = dl / 2
    else false;;
      
(* S(v,d) dla nieparzystej liczby potomkw v,   *)
(* na podstawie d oraz [S(w1,d); ...; S(wp,d)]. *)
let nieparzyste ls d = 
  let dl = length ls
  in
    if pary (tl ls) d then 
      (* Najduszy sznurek moe wychodzi do gry. *)
      (* Szukamy krtszego.                         *)
      let i = ile_par (cut 1 (dl/2) ls) 
	              (rev (cut (dl/2 + 2) dl ls)) d
      in
        if i = dl/2 then 
          (* rodkowy (co do dugoci) sznurek moe *)
	  (* wychodzi do gry. Szukamy krtszego.  *)
	  let j = ile_par (cut (dl/2 + 1) (dl-1) ls) 
	                  (rev (cut 1 (dl/2) ls)) d
	  in 
	    nth ls (j + dl/2) + 1
	else nth ls i + 1
    else 
      (* Nie da si. *)
      inf

(* S(v,d) dla parzystej liczby potomkw v,      *)
(* na podstawie d oraz [S(wp,d); ...; S(w1,d)]. *)
let parzyste ls d =
  if pary ls d then 
    (* Z v wychodzi nowy sznurek. *)
    0
  else 
    if hd ls + 1 <= d then 
      (* Najduszy z wychodzcych sznurkw koczymy, *)
      (* a pozostaych mamy nieparzyst liczb.       *)
      nieparzyste (tl ls) d
    else
      if nieparzyste (tl ls) d <= d then 
        (* Najduszy z wychodzcych sznurkw przechodzi  *)
        (* do gry, o ile pozostae nie s dusze ni d. *)
        hd ls + 1
      else
        (* Nie da si. *)
        inf;;

(* Obliczenie S(v,d) na podstawie d oraz [S(w1,d); ...; S(wp,d)]. *)
let s d l = 

  (* Posortowana nierosnco lista  [S(w1,d); ...; S(wp,d)]. *)
  let l_sort =
    let por x y = if x < y then 1 else if x > y then -1 else 0
    in sort por l

  in
    if length l mod 2 = 1 then 
      (* Nieparzysta liczba potomkw. *)
      nieparzyste l_sort d
    else
      (* Parzysta liczba potomkw. *)
      parzyste l_sort d;;


(* Analiza danego drzewa.           *)
(* Obliczane s:                    *)
(* - wysoko drzewa h,             *)
(* - rednica drzewa sr,            *)
(* - sl = [S(v,0); ... ; S(v, sr)]. *)
let rec analiza (Wezel l) = 
  (* Poddaj analizie poddrzewa. *)
  let w = map analiza l 
  in 

    (* Wysoko i rednica drzewa. *)
    let (h, sr)  = 
      (* Dwie najwiksze wysokoci poddrzew. *)
      let (h1, h2) = 
	fold_left (fun (a1, a2) (h, _, _) -> 
	  if h >= a1 then (h, a1) else 
	  if h >= a2 then (a1, h) else (a1, a2)) 
	  (-1, -1) w
      in 
        (h1 + 1,
	 fold_left (fun a (_,s,_) -> max a s) (h1 + h2 + 2) w)
    in 

      (* [S(v,0); ... ; S(v, sr)] *)
      let sl = 
        (* Iteracja obliczajca S(v, d) dla kolejnych d. *)
        let rec s_iter d ll ak = 
          (* Ogon listy, przy czym jeli pusty, to *)
	  (* dublowana jest gowa.                 *)
          let tail l = 
            match l with 
              [x]  -> [x] |
              h::t -> t |
	      _ -> failwith "tail: l = []"
          in 
            if d > sr then rev ak else 
              s_iter (d+1) 
                (map tail ll) 
                ((s d (map hd ll)) :: ak)
          in 
            s_iter 0 (map (fun (_, _, l) -> l) w)[]

      in (h, sr, sl);;

(* Minimalne ograniczenie na dugo sznurkw. *)
let ograniczenie t = 
  let (_, sr, sl) = analiza t 
  in 
    snd (fold_left (fun (d, m) s -> (d+1, min (max d s) m)) (0, inf) sl) ;;


let (t, n) = wczytaj_dane ()
in 
  begin
    print_int n;
    print_string " ";
    print_int (ograniczenie t);
    print_newline ()
  end;;
  
