/*************************************************************************
 *                                                                       *
 *                       Pogromcy Algorytmw 2004                        *
 *                                                                       *
 *   Rozwizanie zadania: Nawiasy                                        *
 *   Plik:                naw.cpp                                        *
 *   Autor:               Pawe Wolff                                    *
 *************************************************************************/

/* 

   (Poniszy komentarz: PW, 02.10.2003)


   Rozwizanie wzorcowe zadania NAWIASY
   ------------------------------------
   
   Na pocztku algorytmu, na podstawie sowa nawiasowego danego na wejciu,
   budujemy drzewo binarne tak, e:
   - korzeniowi drzewa stowarzyszony jest z caym sowem nawiasowym,
   - jeli wze drzewa stowarzyszony jest z podsowem w caego sowa
     nawiasowego oraz |w| > 1, to jego lewy syn stowarzyszony jest ze slowem v,
     a prawy ze slowem u,
     gdzie w = uv, oraz |u| = |w| div 2, |v| = (|w| + 1) div 2,
   - jeli wze drzewa stowarzyszony jest z podsowem w caego sowa
     nawiasowego oraz |w| = 1, to wze ten jest liciem drzewa.
   Ponadto w wzach drzewa zapisane s dwie liczby:
   - liczba l, czyli liczba niezamknitych nawiasw '('
     w podsowie sowa nawiasowego stowarzyszonego z danym wzem,
   - liczba p, czyli liczba nieotwartych nawiasw ')'
     w podsowie sowa nawiasowego stowarzyszonego z danym wzem.

   Takie drzewo mona zbudowa w czasie O(n); gdzie n = dlugosc slowa
   nawiasowego (patrz funkcja init).
   
   Jeli jeden nawias w sowie nawiasowym zmieniamy na przeciwny,
   to uaktualnienia naszego drzewa mozemy dokona w czasie O(log n)
   (bo log n = wysoko drzewa). (Patrz funkcja change.)
   
   Sprawdzenie, czy aktualne sowo nawiasowe jest poprawnym wyraeniem
   nawiasowym wykonuje si w czasie staym - wystarczy sprawdzi, czy
   liczby l i p zapisane w korzeniu s rwne 0 (patrz funkcja query).

   Zoono czasowa powyszego algorytmu wynosi O(n + s + z * log n),
   gdzie z jest liczb operacji zamiany, a s - liczb operacji sprawdzenia.
   Zoono pamiciowa wynosi O(n).

*/


#include <stdio.h>

#define MAXN 30000

char s[MAXN + 1];
int l[4 * MAXN];
int p[4 * MAXN];
int n, q;

void init(int nr, int il, int ip)
{
  int id = (il+ip)/2;

  if (il < ip) {
    init(2*nr+1, il, id);
    init(2*nr+2, id+1, ip);
    if (p[2*nr+2] >= l[2*nr+1]) {
      l[nr] = l[2*nr+2];
      p[nr] = p[2*nr+1]+p[2*nr+2]-l[2*nr+1];
    } else {
      l[nr] = l[2*nr+1]-p[2*nr+2]+l[2*nr+2];
      p[nr] = p[2*nr+1];
    }
  } else {
    if (s[id] == '(') {
      l[nr] = 1;
      p[nr] = 0;
    } else {
      l[nr] = 0;
      p[nr] = 1;
    }
  }
}

void change(int nr, int il, int ip, int pos)
{
  int id = (il+ip)/2;

  if (il < ip)
  {
    if (pos <= id) {
      change(2*nr+1, il, id, pos);
    } else {
      change(2*nr+2, id+1, ip, pos);
    }
    if (p[2*nr+2] >= l[2*nr+1]) {
      l[nr] = l[2*nr+2];
      p[nr] = p[2*nr+1]+p[2*nr+2]-l[2*nr+1];
    } else {
      l[nr] = l[2*nr+1]-p[2*nr+2]+l[2*nr+2];
      p[nr] = p[2*nr+1];
    }
  } else {
    l[nr] = 1-l[nr];
    p[nr] = 1-p[nr];
  }
}

int query()
{
  return l[0] == 0 && p[0] == 0;
}

int main()
{
  int i, pos;

  scanf("%d %s", &n, s);
  init(0, 0, n-1);
  scanf("%d", &q);
  for (i = 0; i < q; i++) {
    scanf("%d", &pos);
    if (pos == 0)
      printf("%s\n", query() ? "TAK" : "NIE");
    else {
      pos--;
      s[pos] = 1-(s[pos]-'(')+'(';
      change(0, 0, n-1, pos);
    }
  }

  return 0;
}
